/*
 * SPDX-License-Identifier: GPL-3.0-only
 * MuseScore-CLA-applies
 *
 * MuseScore
 * Music Composition & Notation
 *
 * Copyright (C) 2021 MuseScore BVBA and others
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __TABLATURE_H__
#define __TABLATURE_H__

#include <cstddef>
#include <vector>
#include <map>

namespace mu::engraving {
class Chord;
class Note;
class XmlReader;
class XmlWriter;
class Staff;

//---------------------------------------------------------
//   StringData
//---------------------------------------------------------

// defines the string of an instrument
struct instrString {
    instrString(int p = 0, bool o = false, int s = 0)
        : pitch(p), open(o), startFret(s) {}
    int pitch;          // the pitch of the string
    bool open;          // true: string is open | false: string is fretted
    int startFret;      // banjo 5th string starts on 5th fret

    bool operator==(const instrString& d) const { return d.pitch == pitch && d.open == open; }
};

class StringData
{
//      std::vector<int>  stringTable { 40, 45, 50, 55, 59, 64 };   // guitar is default
//      int         _frets = 19;
    std::vector<instrString> stringTable {  };                      // no strings by default
    int _frets = 0;

    static bool bFretting;

    bool        convertPitch(int pitch, int pitchOffset, int* string, int* fret) const;
    int         fret(int pitch, int string, int pitchOffset) const;
    int         getPitch(int string, int fret, int pitchOffset) const;
    void        sortChordNotes(std::map<int, Note*>& sortedNotes, const Chord* chord, int pitchOffset, int* count) const;

public:
    StringData() {}
    StringData(int numFrets, int numStrings, int strings[]);
    StringData(int numFrets, std::vector<instrString>& strings);
    void        set(const StringData& src);
    bool        convertPitch(int pitch, Staff* staff, int* string, int* fret) const;
    int         fret(int pitch, int string, Staff* staff) const;
    void        fretChords(Chord* chord) const;
    int         getPitch(int string, int fret, Staff* staff) const;
    static int  pitchOffsetAt(Staff* staff);
    size_t      strings() const { return stringTable.size(); }
    int         frettedStrings() const;
    const std::vector<instrString>& stringList() const { return stringTable; }
    std::vector<instrString>& stringList() { return stringTable; }
    int         frets() const { return _frets; }
    void        setFrets(int val) { _frets = val; }
    void        read(XmlReader&);
    void        write(XmlWriter&) const;
    bool operator==(const StringData& d) const { return d._frets == _frets && d.stringTable == stringTable; }
    void        configBanjo5thString();
    int         adjustBanjo5thFret(int fret) const;
    bool        isFiveStringBanjo() const;
};
} // namespace mu::engraving
#endif
